{
               Keyboard Controller
               -------------------------------------
               Version      :  1.20
               Unit name    :  Keyboard
               Code name    :  Keyboard Master
               Last Created :  1994/10/21
               Enviroment   :  above XT machine
               Function     :  Control keyboard
               Production   :  KIM, JIN HONG [IZZER]
               Belong       :  PENTAGRAM
               -------------------------------------
}

unit Keyboard;

interface
uses
  dos;

const
  { Keyboard Scan Code }
  _F1=59;  _F7 =65;  _ESC  =1;   _SPACE=57;
  _F2=60;  _F8 =66;  _LEFT =75;  _DEL  =$53;
  _F3=61;  _F9 =67;  _RIGHT=77;  _PAUSE=$E1;
  _F4=62;  _F10=68;  _UP   =72;
  _F5=63;  _F11=87;  _DOWN =80;
  _F6=64;  _F12=88;  _ENTER=28;

type
  MapType = array [byte] of boolean;

var
  KC_Handler:procedure;      {handler for Ctrl+Alt+Del}
  KeyMem    :byte absolute $0040:$0017;
  KeyMap    :MapType;

  FUNCTION  KC_State:boolean;
  PROCEDURE SetBoot(NF:boolean);
  PROCEDURE SetBreak(NF:boolean);

  FUNCTION  PInsert:boolean;
  FUNCTION  PCapsLock:boolean;
  FUNCTION  PNumLock:boolean;
  FUNCTION  PScrollLock:boolean;
  FUNCTION  PALT:boolean;
  FUNCTION  PCTRL:boolean;
  FUNCTION  PLeftShift:boolean;
  FUNCTION  PRightShift:boolean;

  FUNCTION  FKeyPressed:boolean;
  FUNCTION  GetKey:word;
  PROCEDURE PrtSc(ENABLE:boolean);
  PROCEDURE Nothing;
  PROCEDURE Pause;
  PROCEDURE TypematicRate(DEL,RATE:byte);  {Typematic Rate}
  PROCEDURE ClearKeyBuffer;
  PROCEDURE InputCTC(C:string);

implementation
const
  ExtCode   = $e0;
  ExtFlag   : boolean = false;
  Count     : byte    = 0;
  Key_Seg  = $40;
  First    = $1a;
  Last     = $1c;
  BufBegin = $80;
  BufEnd   = $82;
  BreakState:boolean=true;
  BootState :boolean=true;
  Installed :boolean=false;

var
  OldInt09,                {variable for saving old int 9 handler}
  OldExitProc : procedure; {variable for saving old exit proc}
  KeyStat     : ^Byte;     {pointer to keyboard state}

{$F+}
procedure NewInt09(AX,BX,CX,DX,SI,DI,DS,ES,BP : Word); interrupt;
{$F-}
var i,j: Integer;
begin
  asm CLI end;                          {disable interrupts}

  i := Port[$60];                       {get scan code}

  if i = _Pause then Count:=3;
  if Count = 0 then begin
    if i = ExtCode then ExtFlag:=true
    else if ExtFlag then begin
      KeyMap[i or $80]:=i < $80;
      ExtFlag:=false
    end else KeyMap[i and $7f] := i < $80
  end else dec(Count);

  if (i=_Pause) and (not BreakState) then begin
    KeyMap[_Pause]:=true;
    port[$60]:=0;
    i:=0
  end else KeyMap[_Pause]:=false;

  if (not BootState) and ((i and $7F) = _DEL) and ((KeyStat^ and $0C) = $0C) then
    if ((i and $80) = 0) then begin
        j := Port[$61];                 {save kbd status}
        Port[$61] := j and $80;         {reset kbd int}
        Port[$61] := j;                 {set original kbd status}
        Port[$20] := $20;               {signal end of interrupt}
        KC_Handler;                     {execute the handler}
        Port[$20] := $20                {signal end of interrupt}
      end else begin
        j := Port[$61];                 {save kbd status}
        Port[$61] := j and $80;         {reset kbd int}
        Port[$61] := j;                 {set original kbd status}
        Port[$20] := $20                {signal end of interrupt}
      end else OldInt09;                {execute the old int 9}
  asm STI end                           {enable interrupts}
end; {NewInt09}

{$F+}
procedure KC_UnInstall;
{$F-}
begin
  ExitProc := @OldExitProc;            {restore old exit pointer}
  SetIntVec($09,@OldInt09);            {restore old int 9 vector}
  Installed := false                   {set installed flag}
end;

procedure KC_Install;
begin
  if Installed then exit;
  @OldExitProc := ExitProc;
  ExitProc := @KC_UnInstall;
  GetIntVec($09, @OldInt09);
  SetIntVec($09, @NewInt09);
  Installed := true
end;

{$F+}
procedure Nothing;
begin end;
{$F-}

FUNCTION KC_State:boolean;
begin KC_State:=Installed end;

PROCEDURE ClearKeyBuffer;
begin mem[0:$41A]:=mem[0:$41C] end;

PROCEDURE TypematicRate(DEL,RATE:byte); assembler;
asm
  mov   ah, 3
  mov   al, 5
  mov   bh, DEL
  mov   bl, RATE
  int   16h
end;                  {DEL 0..3}  {RATE 0..7}

FUNCTION FKeyPressed:Boolean; Assembler;
ASM
    PUSH DS
    MOV AX, 40h
    MOV DS, AX
    CLI
    MOV AX, [1Ah]
    CMP AX, [1Ch]
    STI
    MOV AX, 0
    JZ @NoPress
    INC AX
    @NoPress:
    POP DS
END;

FUNCTION GetKey:word; assembler;
asm
   mov    ax,Key_Seg
   mov    es,ax
   xor    ax,ax
   mov    si,es:[First]
   cmp    si,es:[Last]
   je     @end
   mov    ax,es:[si]
   add    si,02
   cmp    si,es:[BufEnd]
   jb     @Ok
   mov    si,es:[BufBegin]
   @Ok:
   mov    es:[First],si
   cmp    al,224
   jne    @end
   cmp    ah,70
   jb     @end
   xor    al,al
   @end:
end;

PROCEDURE Pause;
begin
  repeat until GetKey<>0;
  ClearKeyBuffer
end;

procedure InputCTC(C:string);     {Input Character to Consol}
procedure ICC(C:char); assembler;
asm
    mov  cl, C
    mov  ah, 5h
    int  16h
end;
var T:byte;
begin
  for T:=1 to length(C) do ICC(C[T])
end;

procedure PrtSc(ENABLE:boolean);
begin
  mem[$50:$00]:=ord(not ENABLE)
end;

FUNCTION PInsert:boolean;
begin if KeyMem and 128<>0 then PInsert:=true else PInsert:=false end;

FUNCTION PCapsLock:boolean;
begin if KeyMem and 64<>0 then PCapsLock:=true else PCapsLock:=false end;

FUNCTION PNumLock:boolean;
begin if KeyMem and 32<>0 then PNumLock:=true else PNumLock:=false end;

FUNCTION PScrollLock:boolean;
begin if KeyMem and 16<>0 then PScrollLock:=true else PScrollLock:=false end;

FUNCTION PALT:boolean;
begin if KeyMem and 8<>0 then PALT:=true else PALT:=false end;

FUNCTION PCTRL:boolean;
begin if KeyMem and 4<>0 then PCTRL:=true else PCTRL:=false end;

FUNCTION PLeftShift:boolean;
begin if KeyMem and 2<>0 then PLeftShift:=true else PLeftShift:=false end;

FUNCTION PRightShift:boolean;
begin if KeyMem and 1<>0 then PRightShift:=true else PRightShift:=false end;

PROCEDURE SetBoot(NF:boolean);
begin
  BootState:=NF;
  if not NF then begin
    if not KC_State then KC_Install
  end else if KC_State then KC_UnInstall
end;

PROCEDURE SetBreak(NF:boolean);
begin
  BreakState:=NF;
  if not NF then begin
    if not KC_State then KC_Install
  end else if KC_State then KC_UnInstall
end;

begin
  fillchar(KeyMap,sizeof(KeyMap),false);
  KC_Handler:=Nothing;
  KeyStat := Ptr(Seg0040, $17)  { SJM 6/22 -- BP7 protected mode compatible }
end.

